%
%
%  Channel Arithmetics Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Channel Arithmetics" icon="Matlab" tooltip="Create a new channel combining the others using a regular expression.">
%          <Command>MatlabXT::XTChannelArithmetics(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
% 
%
%  Description:
%   
%   Create a new channel combining the others using a regular expression. 
% 

function XTChannelArithmetics(aImarisApplicationID)

% get the application object
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  % called from workspace
  vImarisApplication = aImarisApplicationID;
else
  % connect to Imaris interface
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

vAnswer = inputdlg({sprintf(['Combination expression:\n\n', ...
  'Channel names: ch1, ch2, ...\nUse matlab operators, i.e. ', ...
  '+, -, .*, ./, .^, sqrt, ...\n'])}, ...
    'Channel Arithmetics', 1, {'sqrt(ch1 .* ch2)'});
if isempty(vAnswer), return, end

vDataSet = vImarisApplication.GetDataSet.Clone;

vLastC = vDataSet.GetSizeC;
vDataSet.SetSizeC(vLastC + 1);
vMin = vDataSet.GetChannelRangeMin(0);
vMax = vDataSet.GetChannelRangeMax(0);
vDataSet.SetChannelRange(vLastC, vMin, vMax);

vProgressTotalCount = vDataSet.GetSizeT*vDataSet.GetSizeZ;
vProgressDisplay = waitbar(0, 'Channel Arithmetics');
vProgressCount = 0;

for vTime = 1:vDataSet.GetSizeT
  for vSlice = 1:vDataSet.GetSizeZ
    
    for vChannel = 1:vLastC
      % works on double to allow division and prevent overflows
      eval(sprintf(['ch%i = double(vDataSet.GetDataSliceFloats(', ...
        'vSlice-1, vChannel-1, vTime-1));'], vChannel));
    end
    
    try
      vData = eval(vAnswer{1});
    catch er
      close(vProgressDisplay)
      msgbox(sprintf(['Error while evaluating the expression.\n\n', ...
        'Possible causes: invalid variable names (ch1, ch2, ...), ', ...
        'invalid operators (use .* instead of *)...\n\n', er.message]))
      return
    end

    try
      if strcmp(vDataSet.GetType,'eTypeUInt8')
        vDataSet.SetDataSliceBytes(uint8(vData), vSlice-1, vLastC, vTime-1);
      elseif strcmp(vDataSet.GetType,'eTypeUInt16')
        vDataSet.SetDataSliceShorts(uint16(vData), vSlice-1, vLastC, vTime-1);
      elseif strcmp(vDataSet.GetType,'eTypeFloat')
        vDataSet.SetDataSliceFloats(single(vData), vSlice-1, vLastC, vTime-1);
      end
    catch er
      close(vProgressDisplay)
      msgbox(sprintf(['The result of the expression is not a valid dataset.\n\n', ...
        'Possible causes: invalid result size.\n\n', er.message]))
      return
    end

    vProgressCount = vProgressCount + 1;
    waitbar(vProgressCount/vProgressTotalCount, vProgressDisplay)

  end
end

vImarisApplication.SetDataSet(vDataSet);
close(vProgressDisplay);
